@@ -4,20 +4,27 @@
 
 import csv
 import itertools as itt
+import json
+import warnings
 from collections import defaultdict
 from dataclasses import dataclass, field
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
+    Any,
+    Callable,
     DefaultDict,
     Dict,
+    Iterable,
     List,
     Mapping,
     Optional,
     Sequence,
     Set,
     Tuple,
+    TypeVar,
     Union,
+    cast,
 )
 
 import requests
@@ -34,6 +41,9 @@ __all__ = [
     "chain",
 ]
 
+X = TypeVar("X")
+LocationOr = Union[str, Path, X]
+
 
 @dataclass
 class Record:
@@ -46,7 +56,7 @@ class Record:
     prefix_synonyms: List[str] = field(default_factory=list)
     uri_prefix_synonyms: List[str] = field(default_factory=list)
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         """Check the integrity of the record."""
         for ps in self.prefix_synonyms:
             if ps == self.prefix:
@@ -69,11 +79,11 @@ class Record:
 class DuplicateValueError(ValueError):
     """An error raised with constructing a converter with data containing duplicate values."""
 
-    def __init__(self, duplicates: List[Tuple[Record, Record, str]]):
+    def __init__(self, duplicates: List[Tuple[Record, Record, str]]) -> None:
         """Initialize the error."""
         self.duplicates = duplicates
 
-    def _str(self):
+    def _str(self) -> str:
         s = ""
         for r1, r2, p in self.duplicates:
             s += f"\n{p}:\n\t{r1}\n\t{r2}\n"
@@ -130,6 +140,21 @@ def _get_reverse_prefix_map(records: List[Record]) -> Mapping[str, str]:
     return rv
 
 
+def _prepare(data: LocationOr[X]) -> X:
+    if isinstance(data, Path):
+        with data.open() as file:
+            return cast(X, json.load(file))
+    elif isinstance(data, str):
+        if any(data.startswith(p) for p in ("https://", "http://", "ftp://")):
+            res = requests.get(data)
+            res.raise_for_status()
+            return cast(X, res.json())
+        with open(data) as file:
+            return cast(X, json.load(file))
+    else:
+        return data
+
+
 class Converter:
     """A cached prefix map data structure.
 
@@ -163,7 +188,7 @@ class Converter:
     #: A prefix trie for efficient parsing of URIs
     trie: StringTrie
 
-    def __init__(self, records: List[Record], *, delimiter: str = ":", strict: bool = True):
+    def __init__(self, records: List[Record], *, delimiter: str = ":", strict: bool = True) -> None:
         """Instantiate a converter.
 
         :param records:
@@ -190,7 +215,7 @@ class Converter:
         self.trie = StringTrie(self.reverse_prefix_map)
 
     @classmethod
-    def from_extended_prefix_map_url(cls, url: str, **kwargs) -> "Converter":
+    def from_extended_prefix_map_url(cls, url: str, **kwargs: Any) -> "Converter":
         """Get a converter from a remote JSON file containing an extended prefix map.
 
         :param url: The URL of a JSON file containiing dictionaries corresponding to the :class:`Record` schema
@@ -211,12 +236,13 @@ class Converter:
         >>> url = "https://github.com/biopragmatics/bioregistry/raw/main/exports/contexts/bioregistry.epm.json"
         >>> converter = Converter.from_extended_prefix_map_url(url)
         """
-        res = requests.get(url)
-        res.raise_for_status()
-        return cls.from_extended_prefix_map(res.json(), **kwargs)
+        warnings.warn("directly use Converter.from_extended_prefix_map", DeprecationWarning)
+        return cls.from_extended_prefix_map(url, **kwargs)
 
     @classmethod
-    def from_extended_prefix_map(cls, records, **kwargs) -> "Converter":
+    def from_extended_prefix_map(
+        cls, records: LocationOr[Iterable[Union[Record, Dict[str, Any]]]], **kwargs: Any
+    ) -> "Converter":
         """Get a converter from a list of dictionaries by creating records out of them.
 
         :param records: An iterable of :class:`Record` objects or dictionaries that will
@@ -261,11 +287,18 @@ class Converter:
         >>> converter.compress("https://www.ebi.ac.uk/chebi/searchId.do?chebiId=CHEBI:138488")
         'CHEBI:138488'
         """
-        records = [record if isinstance(record, Record) else Record(**record) for record in records]
-        return cls(records=records, **kwargs)
+        return cls(
+            records=[
+                record if isinstance(record, Record) else Record(**record)
+                for record in _prepare(records)
+            ],
+            **kwargs,
+        )
 
     @classmethod
-    def from_priority_prefix_map(cls, data: Mapping[str, List[str]], **kwargs) -> "Converter":
+    def from_priority_prefix_map(
+        cls, data: LocationOr[Mapping[str, List[str]]], **kwargs: Any
+    ) -> "Converter":
         """Get a converter from a priority prefix map.
 
         :param data:
@@ -296,13 +329,15 @@ class Converter:
                 Record(
                     prefix=prefix, uri_prefix=uri_prefixes[0], uri_prefix_synonyms=uri_prefixes[1:]
                 )
-                for prefix, uri_prefixes in data.items()
+                for prefix, uri_prefixes in _prepare(data).items()
             ],
             **kwargs,
         )
 
     @classmethod
-    def from_prefix_map(cls, prefix_map: Mapping[str, str], **kwargs) -> "Converter":
+    def from_prefix_map(
+        cls, prefix_map: LocationOr[Mapping[str, str]], **kwargs: Any
+    ) -> "Converter":
         """Get a converter from a simple prefix map.
 
         :param prefix_map:
@@ -333,13 +368,15 @@ class Converter:
         return cls(
             [
                 Record(prefix=prefix, uri_prefix=uri_prefix)
-                for prefix, uri_prefix in prefix_map.items()
+                for prefix, uri_prefix in _prepare(prefix_map).items()
             ],
             **kwargs,
         )
 
     @classmethod
-    def from_reverse_prefix_map(cls, reverse_prefix_map: Mapping[str, str]) -> "Converter":
+    def from_reverse_prefix_map(
+        cls, reverse_prefix_map: LocationOr[Mapping[str, str]]
+    ) -> "Converter":
         """Get a converter from a reverse prefix map.
 
         :param reverse_prefix_map:
@@ -370,7 +407,7 @@ class Converter:
         'CHEBI:138488'
         """
         dd = defaultdict(list)
-        for uri_prefix, prefix in reverse_prefix_map.items():
+        for uri_prefix, prefix in _prepare(reverse_prefix_map).items():
             dd[prefix].append(uri_prefix)
         records = []
         for prefix, uri_prefixes in dd.items():
@@ -396,12 +433,11 @@ class Converter:
         >>> "chebi" in Converter.prefix_map
         True
         """
-        res = requests.get(url)
-        res.raise_for_status()
-        return cls.from_reverse_prefix_map(res.json())
+        warnings.warn("directly use Converter.from_reverse_prefix_map", DeprecationWarning)
+        return cls.from_reverse_prefix_map(url)
 
     @classmethod
-    def from_jsonld(cls, data) -> "Converter":
+    def from_jsonld(cls, data: LocationOr[Dict[str, Any]]) -> "Converter":
         """Get a converter from a JSON-LD object, which contains a prefix map in its ``@context`` key.
 
         :param data:
@@ -409,7 +445,7 @@ class Converter:
         :return:
             A converter
         """
-        return cls.from_prefix_map(data["@context"])
+        return cls.from_prefix_map(_prepare(data)["@context"])
 
     @classmethod
     def from_jsonld_url(cls, url: str) -> "Converter":
@@ -426,12 +462,13 @@ class Converter:
         >>> "rdf" in converter.prefix_map
         True
         """
-        res = requests.get(url)
-        res.raise_for_status()
-        return cls.from_jsonld(res.json())
+        warnings.warn("directly use Converter.from_jsonld", DeprecationWarning)
+        return cls.from_jsonld(url)
 
     @classmethod
-    def from_jsonld_github(cls, owner: str, repo: str, *path: str, branch: str = "main"):
+    def from_jsonld_github(
+        cls, owner: str, repo: str, *path: str, branch: str = "main"
+    ) -> "Converter":
         """Construct a remote JSON-LD URL on GitHub then parse with :meth:`Converter.from_jsonld_url`.
 
         :param owner: A github repository owner or organization (e.g., ``biopragmatics``)
@@ -455,7 +492,7 @@ class Converter:
             raise ValueError("final path argument should end with .jsonld")
         rest = "/".join(path)
         url = f"https://raw.githubusercontent.com/{owner}/{repo}/{branch}/{rest}"
-        return cls.from_jsonld_url(url)
+        return cls.from_jsonld(url)
 
     def get_prefixes(self) -> Set[str]:
         """Get the set of prefixes covered by this converter."""
@@ -595,7 +632,7 @@ class Converter:
 
     def file_compress(
         self, path: Union[str, Path], column: int, sep: Optional[str] = None, header: bool = True
-    ):
+    ) -> None:
         """Convert all URIs in the given column of a CSV file to CURIEs.
 
         :param path: A pandas DataFrame
@@ -607,7 +644,7 @@ class Converter:
 
     def file_expand(
         self, path: Union[str, Path], column: int, sep: Optional[str] = None, header: bool = True
-    ):
+    ) -> None:
         """Convert all CURIEs in the given column of a CSV file to URIs.
 
         :param path: A pandas DataFrame
@@ -619,8 +656,12 @@ class Converter:
 
     @staticmethod
     def _file_helper(
-        func, path: Union[str, Path], column: int, sep: Optional[str] = None, header: bool = True
-    ):
+        func: Callable[[str], Optional[str]],
+        path: Union[str, Path],
+        column: int,
+        sep: Optional[str] = None,
+        header: bool = True,
+    ) -> None:
         path = Path(path).expanduser().resolve()
         rows = []
         delimiter = sep or "\t"
@@ -628,7 +669,7 @@ class Converter:
             reader = csv.reader(file_in, delimiter=delimiter)
             _header = next(reader) if header else None
             for row in reader:
-                row[column] = func(row[column])
+                row[column] = func(row[column]) or ""
                 rows.append(row)
         with path.open("w") as file_out:
             writer = csv.writer(file_out, delimiter=delimiter)
@@ -637,7 +678,7 @@ class Converter:
             writer.writerows(rows)
 
 
-def _f(x):
+def _f(x: str) -> str:
     return x
 
 
@@ -654,6 +695,7 @@ def chain(converters: Sequence[Converter], case_sensitive: bool = True) -> Conve
     if not converters:
         raise ValueError
 
+    norm_func: Callable[[str], str]
     if case_sensitive:
         norm_func = _f
     else:
